import logging

from django.core.cache import cache
from django.db.models import Manager, Sum, F

from bank.constants import TERMS_AND_CONDITIONS_DATE

__all__ = ['TransferManager', 'FreezeTransferManager']
logger = logging.getLogger(__name__)


class TransferManager(Manager):
    def get_frozen_balance_eur(self, user_id):
        # FIXME: cyclic import
        from bank.models import Transfer
        frozen_funds = self.filter(
            transfer_type__in=[Transfer.WITHDRAW, Transfer.BUY],
            status__in=[Transfer.PENDING, Transfer.NEW, Transfer.IN_PROGRESS],
            user_id=user_id
        ).aggregate(
            funds=Sum(F('pay_amount') + F('withdraw_fee'))
        )['funds'] or 0
        logger.info(f"User={user_id} frozen balance={frozen_funds} euro")
        return frozen_funds


class FreezeTransferManager(Manager):
    def get_queryset(self):
        # FIXME: cyclic import
        from bank.models import Transfer
        return super().get_queryset().filter(status=Transfer.ON_HOLD)
