import logging
import time
from http.client import RemoteDisconnected

from requests import RequestException as requestsHttpError
from requests.exceptions import BaseHTTPError
from urllib3.exceptions import HTTPError as urlibHttpError

logger = logging.getLogger(__name__)


class BaseRPCClient:
    REPEAT_LIMIT = 10
    DELAY_SEC = 2

    def _wrap_call(self, call_method, *args, **kwargs):
        raised_exception = None
        for i in range(self.REPEAT_LIMIT):
            try:
                return call_method(*args, **kwargs)
            except (RemoteDisconnected, urlibHttpError, requestsHttpError,
                    ConnectionError, BaseHTTPError) as e:
                # In some cases there is just bad connection
                time.sleep(self.DELAY_SEC)
                raised_exception = e
                # Best exception log could be done only in this place
                if i == self.REPEAT_LIMIT - 1:
                    logger.error(e, exc_info=True)

        raise raised_exception

    def _call(self, *args, **kwargs):
        return self._wrap_call(super()._call, *args, **kwargs)
