from urllib import parse as urlparse

from django.conf import settings
from privex.jsonrpc import BitcoinRPC

from interfaces.base_rpc_client import BaseRPCClient


class DashProxy(BaseRPCClient, BitcoinRPC):

    def __init__(self, *args, **kwargs):
        parsed_url = urlparse.urlparse(settings.DASH_NETWORK_URL)
        super().__init__(
            hostname=parsed_url.hostname,
            port=parsed_url.port,
            username=parsed_url.username,
            password=parsed_url.password,
            *args, **kwargs
        )

    def validateaddress(self, address):
        return self.call('validateaddress', address)['isvalid']

    def listreceivedbyaddress(self, minconf=3, include_empty=False, include_watchonly=False, address_filter=None):
        block_height = self.getblockchaininfo()['blocks']
        return [{'txids':self.call('getaddresstxids', {'addresses':[address_filter], 'end': block_height - minconf})}]

    def listunspent(self, minconf=3, maxconf=9999999, addrs=None):
        return self.call('listunspent', minconf, maxconf, addrs)


dash_conn = DashProxy()
