import re
from decimal import Decimal
from typing import Union

from . import dash_conn
from interfaces.constants import DASH
from interfaces.utils import (
    get_received_by_payment_list, calculate_unspend_utxo
)

DASH_DECIMALS = 8


def is_dash_wallet_address(address: str) -> bool:
    """
    Helper method for DASH address validation
    Prefixes:
    X - main network
    y - test network
    """
    dash_address_re = '^(X|y)[1-9A-HJ-NP-Za-km-z]{33}$'

    if not re.search(dash_address_re, address):
        return False
    return True


def dash2duffs(value: Union[Decimal, float]) -> int:
    """
    Function converts dash to duffs(with rounding to 8 digits after point).
    """
    return int(round(float(value) * 10 ** DASH_DECIMALS, DASH_DECIMALS))


def duffs2dash(value: int) -> float:
    """
    Function converts duffs to dash.
    """
    return value / 10 ** DASH_DECIMALS


def get_dash_wallet_balance(address: str) -> Decimal:
    return Decimal(calculate_unspend_utxo(dash_conn, address))


def get_dash_received_by_payment_list(address: str) -> (Decimal, str):
    return get_received_by_payment_list(dash_conn, DASH, address)


def get_entire_dash_wallet_balance() -> Decimal:
    return dash_conn.getbalance(confirmations=3)


def create_dash_wallet() -> str:
    return dash_conn.getnewaddress()


def send_dash(address: str, amount: float) -> str:
    amount = round(amount, DASH_DECIMALS)
    return dash_conn.sendtoaddress(address, amount)
