import json

from django.conf import settings
from web3.datastructures import AttributeDict
from web3.eth import Eth
from web3.net import Net
from web3.version import Version
from web3.geth import Geth, GethAdmin, GethPersonal, GethTxPool


GAS_COEF = 1

ETH_NETWORKS = {
    'mainnet': AttributeDict({
        'id': 1,
        'name': 'Main',
        'contract': settings.MAINNET_SMART_CONTRACT_ACCOUNT,
        'api': 'api',
        'url': ''
    }),
    'rinkeby': AttributeDict({
        'id': 2,
        'name': 'Rinkeby',
        'contract': settings.RINKEBY_SMART_CONTRACT_ACCOUNT,
        'api': 'api-rinkeby',
        'url': 'rinkeby.'
    }),
}

CONTRACT_ABI = json.dumps([
    {
        "constant": False,
        "inputs": [
            {
                "internalType": "address[]",
                "name": "addresses",
                "type": "address[]"
            },
            {
                "internalType": "uint256[]",
                "name": "amounts",
                "type": "uint256[]"
            }
        ],
        "name": "distribute",
        "outputs": [],
        "payable": True,
        "stateMutability": "payable",
        "type": "function"
    },
    {
        "payable": True,
        "stateMutability": "payable",
        "type": "fallback"
    }
])

DEFAULT_MODULES = {
    "eth": (Eth,),
    "net": (Net,),
    "version": (Version,),
    "geth": (Geth, {
        "admin": (GethAdmin,),
        "personal": (GethPersonal,),
        "txpool": (GethTxPool,),
    }),
}

STRATEGY_FAST = 'fast'
STRATEGY_SLIGHTLY_MEDIUM = 'slightly_medium'
STRATEGY_MEDIUM = 'medium'
STRATEGY_SLOW = 'slow'
STRATEGY_GLACIAL = 'glacial'
