from django.conf import settings

from web3.middleware import geth_poa_middleware
from web3 import Web3
from web3.eth import Eth
from constance import config

from interfaces.ethereum.middleware import (
    PlatformFailedRequestNotificationMiddleware,
    PlatformRetrySendRequestMiddleware,
    DailyPaymentsStrategyMiddleware
)
from interfaces.ethereum.constants import DEFAULT_MODULES
from interfaces.ethereum.strategies import get_strategy


class KuailianEth(Eth):

    def generateGasPrice(self, transaction_params=None):
        self.gasPriceStrategy = get_strategy(
            config.GAS_STRATEGY_DAILY_PAYMENTS
        )
        return super().generateGasPrice(transaction_params)


default_modules = DEFAULT_MODULES
default_modules["eth"] = (KuailianEth,)


web3_geth = Web3(
    Web3.HTTPProvider(settings.GETH_NODE_URL, request_kwargs={'timeout': 30}),
    modules=default_modules
)

if settings.GETH_NETWORK == 'rinkeby':
    web3_geth.middleware_onion.inject(geth_poa_middleware, layer=0)

web3_geth.middleware_onion.add(PlatformFailedRequestNotificationMiddleware)
web3_geth.middleware_onion.add(PlatformRetrySendRequestMiddleware)

web3_geth.middleware_onion.remove('gas_price_strategy')
web3_geth.middleware_onion.add(DailyPaymentsStrategyMiddleware)
