from django.conf import settings

from web3 import Web3
from web3.providers.auto import load_provider_from_uri
from web3.auto.infura.endpoints import build_infura_url, build_http_headers
from web3.middleware import geth_poa_middleware
from web3.eth import Eth
from constance import config

from interfaces.ethereum.middleware import InvoicesStrategyMiddleware
from interfaces.ethereum.constants import DEFAULT_MODULES
from interfaces.ethereum.strategies import get_strategy


class KuailianEth(Eth):

    def generateGasPrice(self, transaction_params=None):
        self.gasPriceStrategy = get_strategy(config.GAS_STRATEGY_INVOICES)
        return super().generateGasPrice(transaction_params)


default_modules = DEFAULT_MODULES
default_modules["eth"] = (KuailianEth,)


_headers = build_http_headers()
_infura_url = build_infura_url(f'{settings.INFURA_NETWORK}.infura.io')
web3_infura = Web3(
    load_provider_from_uri(_infura_url, _headers), modules=default_modules
)

if settings.INFURA_NETWORK == 'rinkeby':
    web3_infura.middleware_onion.inject(geth_poa_middleware, layer=0)

web3_infura.middleware_onion.remove('gas_price_strategy')
web3_infura.middleware_onion.add(InvoicesStrategyMiddleware)
