from django.conf import settings

from web3.middleware import geth_poa_middleware
from web3 import Web3
from web3.eth import Eth
from constance import config

from interfaces.ethereum.strategies import get_strategy
from interfaces.ethereum.constants import DEFAULT_MODULES
from interfaces.ethereum.middleware import (
    BankFailedRequestNotificationMiddleware,
    BankRetrySendRequestMiddleware,
    BankStrategyMiddleware
)


class KuailianEth(Eth):

    def generateGasPrice(self, transaction_params=None):
        self.gasPriceStrategy = get_strategy(config.GAS_STRATEGY_BANK)
        return super().generateGasPrice(transaction_params)


default_modules = DEFAULT_MODULES
default_modules["eth"] = (KuailianEth,)


web3_kraken = Web3(
    Web3.HTTPProvider(settings.GETH_KRAKEN_NODE_URL, request_kwargs={'timeout': 30}),
    modules=default_modules
)

if settings.GETH_NETWORK == 'rinkeby':
    web3_kraken.middleware_onion.inject(geth_poa_middleware, layer=0)

web3_kraken.middleware_onion.add(BankFailedRequestNotificationMiddleware)
web3_kraken.middleware_onion.add(BankRetrySendRequestMiddleware)
web3_kraken.middleware_onion.remove('gas_price_strategy')
web3_kraken.middleware_onion.add(BankStrategyMiddleware)
