from web3 import Web3
from web3.gas_strategies.time_based import (
    construct_time_based_gas_price_strategy, fast_gas_price_strategy,
    medium_gas_price_strategy, slow_gas_price_strategy,
    glacial_gas_price_strategy
)
from web3.types import GasPriceStrategy

from interfaces.ethereum.constants import (
    STRATEGY_FAST, STRATEGY_SLIGHTLY_MEDIUM, STRATEGY_SLOW, STRATEGY_GLACIAL
)


def get_strategy(strategy: str) -> GasPriceStrategy:
    if strategy == STRATEGY_FAST:
        # fast: mine within 1 minute
        return fast_gas_price_strategy
    elif strategy == STRATEGY_SLIGHTLY_MEDIUM:
        # slightly_medium: mine within 3 minutes
        return construct_time_based_gas_price_strategy(max_wait_seconds=40)
    elif strategy == STRATEGY_SLOW:
        # slow: mine within 1 hour (60 minutes)
        return slow_gas_price_strategy
    elif strategy == STRATEGY_GLACIAL:
        # glacial: mine within the next 24 hours
        return glacial_gas_price_strategy
    else:
        # medium: mine within 10 minutes
        return medium_gas_price_strategy


def set_strategy(web3: Web3, strategy: str) -> Web3:
    web3.eth.setGasPriceStrategy(get_strategy(strategy))
    return web3
