import logging
from typing import Union, List, Dict, Any
from urllib.parse import urlencode, urljoin

import requests
from django.conf import settings
from services_connector.core.handlers import (
    network_error_handler, response_handler
)
from interfaces.constants import BTC, LTC, DASH, BCH

__all__ = ['BtxExplorerApi', ]
logger = logging.getLogger(__name__)


class BtxExplorerApi:
    BTX_CURRENCY_MAPPING = {
        BTC: 1,
        LTC: 2,
        DASH: 3,
        BCH: 4,
    }

    @network_error_handler(timeout=2)
    def get_transactions(
        self, query: Dict[str, Any]
    ) -> Union[List[Dict[str, Union[str, int, List[Dict[str, Union[str, int]]]]]], None]:
        """
        Get transactions which was made from wallet to internal service wallets.

        :param query: A request query parameters
        :type query: str
        :return: transactions which was made from wallet
        :rtype: Union[List[Dict[str, Union[str, int, List[Dict[str, Union[str, int]]]]]], None]
        """
        query = urlencode(query)

        msg = f"get transaction by {query} from btx-explorer"
        logger.info(f"Start {msg}")

        url = urljoin(
            urljoin(settings.BTX_EXPLORER_API_URL, "api/transactions/"),
            f"?{query}"
        )

        response = requests.get(url=url, timeout=settings.EXPLORERS_API_TIMEOUT)
        is_error, data = response_handler(response=response, msg_text=msg)

        if is_error:
            return None
        else:
            return data
