import logging
import time

from django.conf import settings
from web3.exceptions import TransactionNotFound

from interfaces.ethereum import web3_geth, web3_kraken

logger = logging.getLogger(__name__)

TANK_OPTION = 'tank'
ERC20_HELPER_OPTION = 'erc20_helper'


def request_eth_from_wallet(receiver, value, from_wallet='tank', requester=''):
    if from_wallet == TANK_OPTION:
        wallet_address = settings.TANK_WALLET_ADDRESS
        wallet_passphrase = settings.TANK_WALLET_PASSPHRASE
        w3 = web3_geth
    elif from_wallet == ERC20_HELPER_OPTION:
        wallet_address = settings.KRAKEN_WITHDRAW_WALLET_ETH
        wallet_passphrase = settings.KRAKEN_WITHDRAW_WALLET_ETH_PASSPHRASE
        w3 = web3_kraken
    else:
        logger.error(f'{requester}: Try to request eth from wrong wallet.')
        raise ValueError(f'{requester}: Try to request eth from wrong wallet.')

    logger.info(f'{requester}: requested amount from {from_wallet} wallet: {value}')
    gas_price = web3_geth.eth.generateGasPrice()
    w3.geth.personal.unlockAccount(
        wallet_address, wallet_passphrase
    )
    tank_tx = w3.eth.sendTransaction(
        {
            'to': receiver,
            'from': wallet_address,
            'value': value,
            'gasPrice': gas_price
        }
    )
    tx_hex = w3.toHex(tank_tx)
    if tx_hex.startswith('0x'):
        while True:
            try:
                rec = w3.eth.getTransactionReceipt(
                    tx_hex
                )
                if rec.status == 1:
                    break
                else:
                    logger.error(f'{requester}: {from_wallet} transaction is failed')
                    raise ValueError(f'{requester}: {from_wallet} transaction is failed')
            except TransactionNotFound:
                time.sleep(15)
                continue
    else:
        message = f'{requester}: Can not unlock {from_wallet} wallet'
        logger.error(message)
        raise ValueError(message)
