from decimal import Decimal
from typing import Union

import ltc_conn
from interfaces.constants import LTC
from interfaces.utils import (
    get_received_by_payment_list, calculate_unspend_utxo
)

LTC_DECIMALS = 8


def is_ltc_wallet_address(address: str) -> bool:
    """
    Unfortunately there is no library found that checks ltc address correct
    For example this one should be valid "ltc1q3ankncfgtx4d8062gk9wgw420r54kcp3q3y9q0"
    But libs I found failed with it.
    It is possible to use regex, but it doesn't checks checksums
    However this check is valid only with the address on the same network.
    If we send there test address to mainnet node, it returns False
    """
    return ltc_conn.validateaddress(address)


def ltc2photons(value: Union[Decimal, float]) -> int:
    """
    Function converts btc to satoshi(with rounding to 8 digits after point).
    """
    return int(round(float(value) * 10 ** LTC_DECIMALS, LTC_DECIMALS))


def photons2ltc(value: int) -> float:
    """
    Function converts satoshi to btc.
    """
    return value / 10 ** LTC_DECIMALS


def get_ltc_wallet_balance(address: str) -> Decimal:
    return Decimal(calculate_unspend_utxo(ltc_conn, address))


def get_ltc_received_by_payment_list(address: str) -> (Decimal, str):
    return get_received_by_payment_list(ltc_conn, LTC, address)


def get_entire_ltc_wallet_balance() -> Decimal:
    return ltc_conn.getbalance(confirmations=3)


def create_ltc_wallet() -> str:
    return ltc_conn.getnewaddress()


def send_ltc(address: str, amount: float) -> str:
    amount = round(amount, LTC_DECIMALS)
    return ltc_conn.sendtoaddress(address, amount)
