import logging
import requests
from typing import Union, List, Dict, Any
from urllib.parse import urljoin, urlencode

from django.conf import settings
from services_connector.core.handlers import (
    network_error_handler, response_handler
)

logger = logging.getLogger(__name__)
__all__ = ['XrpExplorerApi', ]


class XrpExplorerApi:
    @network_error_handler(timeout=2)
    def get_transactions(
        self, query: Dict[str, Any]
    ) -> Union[List[Dict[str, Union[str, int, List[Dict[str, Union[str, int]]]]]], None]:
        """
        Get transactions which was made from wallet to internal service wallets.

        :param query: A request query parameters
        :type query: Dict
        :return: transactions which was made from invoice wallet
        :rtype: Union[List[Dict[str, Union[str, int, List[Dict[str, Union[str, int]]]]]], None]
        """
        query_params = urlencode(query)
        msg = f"get transaction by {query_params} from xrp-explorer"
        logger.info(f"Start {msg}")

        url = urljoin(
            urljoin(settings.XRP_EXPLORER_API_URL, "api/transactions/"),
            f"?{query_params}"
        )
        response = requests.get(url=url, timeout=settings.EXPLORERS_API_TIMEOUT)
        is_error, data = response_handler(response=response, msg_text=msg)

        if is_error:
            return None
        else:
            return data
