from decimal import Decimal
from typing import Union

from django.utils.crypto import get_random_string

from interfaces.ethereum.helpers import is_eth_wallet_address
from interfaces.ethereum import web3_infura
from interfaces.tether import tether_contract
from invoices.models import Wallet


def is_tether_wallet_address(address: str) -> bool:
    """
    Helper method for Tether address validation (same as for bitcoin)
    """
    return is_eth_wallet_address(address)


def usdt2microcents(value: Union[Decimal, float]) -> int:
    """
    Function converts usdt to microcents(with rounding to 8 digits after point).
    """
    return int(round(float(value) * 1e6, 6))


def microcents2usdt(value: int) -> float:
    """
    Function converts microcents to usdt.
    """
    return value / 1e6


def get_usdt_wallet_balance(address: str) -> Decimal:
    balance = tether_contract.get_balance(address)
    return Decimal(microcents2usdt(balance))


def create_usdt_wallet(fake: bool = False) -> Wallet:
    password = get_random_string(length=20)
    wallet = web3_infura.eth.account.create(password)
    wallet_obj = Wallet.objects.create(
        address=wallet.address,
        password=password,
        private_key=web3_infura.toHex(wallet.privateKey),
        cryptocurrency=Wallet.USDT,
        fake=fake
    )
    return wallet_obj


def send_usdt(wallet_from, _to: str, amount: float, gas_price: int = None, nonce: int = None, private_key=None) -> str:
    return tether_contract.transfer(wallet_from, _to, usdt2microcents(amount),
                                  gas_price=gas_price,
                                  nonce=nonce,
                                  private_key=private_key)
