from decimal import Decimal
from typing import Union

from django.utils.crypto import get_random_string

from interfaces.ethereum import web3_infura
from interfaces.usdc import usdc_contract
from invoices.models import Wallet


def usdc2mircousd(value: Union[Decimal, float]) -> int:
    """
    Function converts usdc to microdollars
    """
    return int(round(float(value) * 1e6, 6))


def mircousd2usdc(value: int) -> float:
    """
    Function converts microdollars to usdc.
    """
    return value / 1e6


def get_usdc_wallet_balance(address: str) -> Decimal:
    balance = usdc_contract.get_balance(address)
    return Decimal(mircousd2usdc(balance))


def create_usdc_wallet(fake: bool = False) -> Wallet:
    password = get_random_string(length=20)
    wallet = web3_infura.eth.account.create(password)
    wallet_obj = Wallet.objects.create(
        address=wallet.address,
        password=password,
        private_key=web3_infura.toHex(wallet.privateKey),
        cryptocurrency=Wallet.USDC,
        fake=fake
    )
    return wallet_obj


def send_usdc(wallet_from, _to: str, amount: float, gas_price: int = None, nonce: int = None, private_key=None):
    return usdc_contract.transfer(wallet_from, _to, usdc2mircousd(amount),
                                  gas_price=gas_price,
                                  nonce=nonce,
                                  private_key=private_key)
