from datetime import datetime, timedelta, date

from django.db.models import Manager
from django.utils.timezone import make_aware


class EthExchangeRateManager(Manager):

    def get_queryset(self):
        return super().get_queryset().order_by('-created')

    def get_usd_rate_for_date(self, for_date):
        if isinstance(for_date, date):
            for_date = make_aware(datetime.combine(for_date, datetime.min.time()))
        exchange_rate_usd_query = self.model.objects.filter(
            provider=self.model.COINGECKO,
            exchange_type=self.model.USD,
            created__lte=for_date,
        )

        if exchange_rate_usd_query.count() == 0:
            exchange_rate_usd_query = self.model.objects.filter(exchange_type=self.model.USD)

        exchange_rate_usd_obj = None
        exchange_rate_usd = None

        if exchange_rate_usd_query.count() > 0:
            exchange_rate_usd_obj = exchange_rate_usd_query.first()
            exchange_rate_usd = exchange_rate_usd_obj.rate

        return exchange_rate_usd, exchange_rate_usd_obj

    def get_btc_rate_for_date(self, for_date):
        if isinstance(for_date, date):
            for_date = make_aware(datetime.combine(for_date, datetime.min.time()))
        exchange_rate_btc_query = self.model.objects.filter(
            provider=self.model.COINGECKO,
            exchange_type=self.model.BTC,
            created__lte=for_date,
        )

        if exchange_rate_btc_query.count() == 0:
            exchange_rate_btc_query = self.model.objects.filter(
                provider=self.model.COINGECKO,
                exchange_type=self.model.BTC
            )

        exchange_rate_btc = None
        exchange_rate_btc_obj = None

        if exchange_rate_btc_query.count() > 0:
            exchange_rate_btc_obj = exchange_rate_btc_query.first()
            exchange_rate_btc = exchange_rate_btc_obj.rate

        return exchange_rate_btc, exchange_rate_btc_obj

    def get_rates_for_payment_orders(self, for_date) -> dict:
        if isinstance(for_date, date):
            for_date = make_aware(datetime.combine(for_date, datetime.min.time()))

        exchange_types = [
            self.model.USD,
            self.model.BTC,
            self.model.ETH_EUR
        ]

        exchange_rates = self.model.objects.filter(
            exchange_type__in=exchange_types,
            created__lte=for_date,
        ).distinct('exchange_type').order_by('exchange_type', '-created')

        rates = dict()
        for rate in exchange_rates:
            rates[rate.exchange_type] = rate

        return rates
